# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 2024/11/01 17:01
@Description: Description
@File: system_env_api.py
"""
from api_requester.http.eclinical_requests import *


class CommonSystemEnvApi:
    """
    The CommonSystemEnvApi was generated by Generator from the SystemEnvController.java file.
    """

    @get('/system/env')
    def get_current_system_env_api(self, *args, **kwargs): ...

    @get('/system/env/list')
    def sponsor_env_list_api(self, *args, **kwargs): ...

    @put('/system/env/{envId}/role/{roleId}')
    def switch_env_api(self, *args, **kwargs): ...



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 8/22/2024 11:44 AM
@Description: Description
@File: run.py
"""
import ast

import datetime
import json
import os
from typing import Any


def sort_json_data(data: Any) -> Any:
    """
    Recursively sort JSON-like data (dictionaries and lists) and return the sorted data.

    :param data: JSON-like data (dict, list, or primitive).
    :return: Sorted JSON-like data.
    """

    def sort_key(item: Any) -> Any:
        if isinstance(item, dict):
            return tuple(sorted(item.items()))
        elif isinstance(item, list):
            return tuple(sort_key(subitem) for subitem in item)
        else:
            return item

    if isinstance(data, dict):
        # 对字典进行递归排序
        return {k: sort_json_data(v) for k, v in sorted(data.items())}
    elif isinstance(data, list):
        # 对列表进行排序，列表中的每一项也是递归排序的
        return sorted([sort_json_data(item) for item in data], key=sort_key)
    elif isinstance(data, str):
        try:
            parsed_data = ast.literal_eval(data)
            return sort_json_data(parsed_data)
        except (ValueError, SyntaxError):
            return data
    else:
        return data


def are_json_equal(str1, str2):
    """
    Compare two JSON strings by parsing, sorting, and then comparing the data.

    :param str1: First JSON string.
    :param str2: Second JSON string.
    :return: True if both JSON strings represent equivalent data, False otherwise.
    """
    try:
        data1 = json.loads(str1)
        data2 = json.loads(str2)
    except (json.JSONDecodeError, TypeError):
        raise ValueError("Invalid JSON string provided.")
    return sort_json_data(data1) == sort_json_data(data2)


class ComplexEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        if isinstance(obj, datetime.date):
            return obj.strftime('%Y-%m-%d')
        if isinstance(obj, bytes):
            # return str(obj, encoding='utf-8')
            return bytes.decode(obj)
        return json.JSONEncoder.default(self, obj)


def to_json_file(base_path, file_name, obj, sort_keys=True, indent=4):
    if not obj:
        return
    os.makedirs(base_path, exist_ok=True)
    path = f"{base_path}/{file_name}.json"
    with open(path, "w") as fp:
        json.dump(obj, fp, indent=indent, sort_keys=sort_keys, cls=ComplexEncoder)



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 3/18/2024 4:59 PM
@Description: Description
@File: log.py
"""
import logging
import os
import re
import time

from api_requester.utils.path import root
from api_requester.utils.time_utils import now_yyyy_mm_dd


class Logger:
    """
   define log
    """

    def __init__(self, path=None, c_level=logging.INFO, f_level=logging.DEBUG):
        if path is None:
            path = os.path.join(root(), "logs", f"{now_yyyy_mm_dd(time.time())}.log")
        self.path = path
        self.sh = None
        self.fh = None
        # create log file in case of no file
        if not os.path.exists(path):
            path_dir = os.path.dirname(path)
            if not os.path.exists(path_dir):
                os.makedirs(path_dir)
            file = open(path, "w", encoding="utf-8")
            file.close()
        # create logger
        self.logger = logging.getLogger(path)
        self.logger.setLevel(logging.DEBUG)
        # in case of creating multi-logger object
        if not self.logger.handlers:
            # set log format
            fmt = logging.Formatter("[%(asctime)s.%(msecs)03d] [thread:%(thread)d] [threadName:%(threadName)s] "
                                    "[%(levelname)s %(message)s]", datefmt=r"%Y-%m-%d %H:%M:%S")
            # %(filename)s [line:%(lineno)d]
            # set CMD log
            self.sh = logging.StreamHandler()
            self.sh.setFormatter(fmt)
            self.sh.setLevel(c_level)
            # set log in file
            self.fh = logging.FileHandler(path, encoding="utf-8")
            self.fh.setFormatter(fmt)
            self.fh.setLevel(f_level)
            self.logger.addHandler(self.sh)
            self.logger.addHandler(self.fh)

    def __del__(self):
        try:
            if self.sh is not None:
                self.logger.removeHandler(self.sh)
            if self.fh is not None:
                self.logger.removeHandler(self.fh)
        finally:
            pass

    def debug(self, message):
        self.logger.debug(self._msg(message))

    def info(self, message):
        self.logger.info(self._msg(message))

    def warn(self, message):
        self.logger.warning(self._msg(message))

    def error(self, message):
        self.logger.error(self._msg(message))

    def cri(self, message):
        self.logger.critical(self._msg(message))

    @staticmethod
    def _msg(message):
        try:
            if "SQL>>>" in message:
                b = re.compile(r"\s{2,}")
                message = b.sub(" ", message)
                return message
            elif re.search(r"\n\n", message):
                message = re.sub(r"\n\n", r"\n", message)
                return message
        finally:
            return message



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 3/18/2024 4:59 PM
@Description: Description
@File: constant.py
"""
from enum import Enum, unique


@unique
class AppEnum(Enum):

    def __init__(self, system_id, code, description):
        self.id = system_id
        self.code = code
        self.description = description

    ADMIN = (1, "admin", "ADMIN")
    CTMS = (2, "ctms", "CTMS")
    ETMF = (3, "etmf", "eTMF")
    DESIGN = (4, "design", "DESIGN")
    EDC = (5, "edc", "EDC")
    IWRS = (6, "iwrs", "IWRS")
    E_CONSENT = (7, "econsent", "eConsent")
    PV = (8, "pv", "PV")
    CODING = (10, "coding", "CODING")
    IMAGING = (11, "imaging", "Eclinical IMAGING System")
    CMD = (12, "cmd", "Eclinical CMD System")
    IRC = (13, "irc", "Eclinical IRC System")


class UserType(Enum):

    def __init__(self, code, type_name):
        self.code = code
        self.type_name = type_name

    user = (1, "User")
    account = (2, "Account")


@unique
class AppEnvEnum(Enum):

    def __init__(self, code, description):
        self.code = code
        self.description = description

    DEV = ("dev", "DEV")
    UAT = ("uat", "UAT")
    PROD = ("prod", "PROD")


@unique
class BizSqlType(Enum):

    def __init__(self, code, description):
        self.code = code
        self.description = description

    INITIAL = (1, "initial")
    INCREMENTAL = (2, "incremental")


class TestEnvType(str):
    local = "local"
    us_dev = "us.dev"
    us_demo = "us.demo"


@unique
class TestEnv(Enum):

    def __init__(self, code, ttype):
        self.code = code
        self.ttype = ttype

    us_dev = ("us.dev", TestEnvType.us_dev)
    us_demo = ("us.demo", TestEnvType.us_demo)
    dev03 = ("dev03", TestEnvType.local)
    dev04 = ("dev04", TestEnvType.local)
    dev01 = ("dev01", TestEnvType.local)
    dev02 = ("dev02", TestEnvType.local)
    test01 = ("test01", TestEnvType.local)

    @classmethod
    def from_code(cls, code):
        for member in cls:
            if member.code == code:
                return member
        return None

    def __str__(self):
        return self.code


@unique
class RoleEnum(Enum):

    def __init__(self, code, ttype=None):
        self.code = code
        self.ttype = ttype

    DM = ("DM",)
    CRC = ("CRC",)
    CRA = ("CRA",)



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 3/18/2024 4:59 PM
@Description: Description
@File: time_utils.py
"""

import datetime
import time


def now(timestamp):
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(timestamp))


def now_yyyy_mm_dd(timestamp):
    return time.strftime("%Y-%m-%d", time.localtime(timestamp))


def now_utc(timestamp):
    return datetime.datetime.utcfromtimestamp(timestamp)


def now_for_file(timestamp):
    return time.strftime("%Y-%m-%d %H-%M-%S", time.localtime(timestamp))



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 2024/03/19 13:55
@Description: Description
@File: user_on_board_application_service_impl.py
"""
from api_requester.core.admin.service.user_on_board_application_service import AdminUserOnBoardApplicationService
from api_requester.utils.lib import get_val_from_list


class AdminUserOnBoardApplicationServiceImpl(AdminUserOnBoardApplicationService):
    """
    AdminUserOnBoardApplicationServiceImpl
    """

    def __init__(self):
        AdminUserOnBoardApplicationService.__init__(self)

    @get_val_from_list()
    def get_company_id(self, name=None):
        return self.get_company_list()



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 9/13/2024 5:45 PM
@Description: Description
@File: token_manager.py
"""
from api_requester.dto.biz_base import BizBase
from api_requester.http.sample_headers import SampleHeaders


def apply_token_to_headers(instance):
    """
     从实例的响应头中提取 token，并更新到实例的请求头中。

    :param instance: 继承自 BizBase 的实例，包含响应结果和请求头信息。
    :return: 无返回值。若存在 token，则更新实例的请求头。
    :return:
    """
    if not isinstance(instance, BizBase):
        return
    instance: BizBase
    if instance.last_result is None:
        return
    token = instance.last_result.headers.get("token")
    if token:
        headers = SampleHeaders()
        headers.add_authorization(token)
        instance.headers = headers.to_h()



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 2024/11/01 17:01
@Description: Description
@File: system_env_service.py
"""
from api_requester.core.common.api.system_env_api import CommonSystemEnvApi


class CommonSystemEnvService(CommonSystemEnvApi):
    """
    The CommonSystemEnvService was generated by Generator from the SystemEnvController.java file.
    """

    def __init__(self):
        CommonSystemEnvApi.__init__(self)

    def get_current_system_env(self):
        return self.get_current_system_env_api()

    def sponsor_env_list(self):
        return self.sponsor_env_list_api()

    def switch_env(self, env_id: int, role_id: int):
        return self.switch_env_api(url_kwargs=dict(envId=env_id, roleId=role_id))



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 7/24/2023 3:48 PM
@Description: Description
@File: gateway.py
"""
import os.path
import re
from urllib.parse import urlparse

from requests.structures import LookupDict

from api_requester.utils.constant import AppEnum
from api_requester.utils.path import docs_path
from api_requester.utils.read_file import connect_to

_api = {
    AppEnum.ADMIN.code: (AppEnum.ADMIN.code,),
    AppEnum.EDC.code: (AppEnum.EDC.code,),
    AppEnum.CTMS.code: (AppEnum.CTMS.code,),
    AppEnum.ETMF.code: (AppEnum.ETMF.code,),
    AppEnum.DESIGN.code: ('designer', AppEnum.DESIGN.code),
    AppEnum.IWRS.code: (AppEnum.IWRS.code,),
    "external": ('external',),
    AppEnum.CODING.code: (AppEnum.CODING.code,),
    AppEnum.IMAGING.code: (AppEnum.IMAGING.code,),
    AppEnum.PV.code: (AppEnum.PV.code,),
}
apis = LookupDict(name='api')


def _init():
    for app_api, apps in _api.items():
        for app in apps:
            setattr(apis, app, app_api)


_init()


class Gateway(object):
    def __init__(self, test_env):
        self.test_env = test_env
        yaml_path = os.path.join(docs_path(), "application.yaml")
        self.api_yaml = connect_to(yaml_path).data

    def _url(self, app_api, api, external=False):
        server_url = self._server_url(app_api)
        api_url = self._ex_domain(app_api, api) or self._in_domain(app_api, api, external)
        return "{0}{1}".format(server_url, api_url)

    def _server_url(self, app_api):
        server_url_mapping = self.api_yaml["serverUrl"]
        server_url = server_url_mapping.get(self.test_env)
        if server_url is not None:
            return server_url
        else:
            postfix = dict(portal_api=".admin", ctms_api=".ctms", designer_api=".designer",
                           etmf_api=".etmf", edc_api=".edc", iwrs_api=".iwrs").get(app_api)
            return server_url_mapping.get("{0}{1}".format(self.test_env, postfix))

    def _in_domain(self, app_api, api, external=False):
        domain = self.api_yaml.get(app_api).get("urlPrefix")
        api_url = self._disposed(app_api, api) or self._not_disposed(api)
        return api_url if (external and domain in api_url) else "{0}{1}".format(domain, api_url)

    def _ex_domain(self, app_api, api):
        api_url = self._disposed(app_api, api) or self._not_disposed(api)
        if "local" in self.test_env:
            return api_url
        return None

    def _disposed(self, app_api, api):
        return self.api_yaml.get(app_api).get(api)

    @staticmethod
    def _not_disposed(api):
        return api

    @staticmethod
    def get_app_api(app):
        return eval(f"apis.{app.lower()}")

    def url(self, app, api, external=False):
        # if external:  # s3前端迁移
        #     self.test_env = f"{self.test_env}.external"
        tmp_url = self._url(self.get_app_api(external is False and app or "external"), api, external)
        return self.rewrite_path(app, tmp_url)

    def netloc(self, app):
        server_url = self._server_url(self.get_app_api(app))
        return urlparse(server_url).netloc

    def rewrite_path(self, app, api):
        app_config = self.api_yaml.get(app)
        filters = app_config.get("filters")
        if filters is not None:
            for item in filters:
                if item.startswith("RewritePath="):
                    item = item.replace("RewritePath=", "")
                    rewrite_rule, target = item.split(",")
                    api = re.sub(rewrite_rule, target, api)
        return api



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 7/24/2023 3:48 PM
@Description: Description
@File: authorize.py
"""
import time

from api_requester.core.admin.service.auth_service import AdminAuthService
from api_requester.core.admin.service.impl.system_env_service_impl import CommonSystemEnvServiceImpl
from api_requester.core.admin.service.impl.user_on_board_application_service_impl import \
    AdminUserOnBoardApplicationServiceImpl
from api_requester.dto.admin.cross_user_user_on_board_dto import CrossUserUserOnBoardDto
from api_requester.dto.admin.jwt_authentication_request import JwtAuthenticationRequest
from api_requester.dto.admin.user_on_board_dto import UserOnBoardDto
from api_requester.dto.biz_base import BizBase
from api_requester.dto.user import EClinicalUser
from api_requester.http.sample_headers import SampleHeaders
from api_requester.utils.constant import AppEnum, UserType
from api_requester.utils.log import Logger
from api_requester.utils.rsa import encrypt_password


class Authorize(BizBase, SampleHeaders, AdminAuthService, AdminUserOnBoardApplicationServiceImpl,
                CommonSystemEnvServiceImpl):

    def __init__(self, user: EClinicalUser):
        BizBase.__init__(self)
        self.user = user
        self.login_app = self.user.app
        self.user.app = AppEnum.ADMIN.code
        self.time_mills = time.time()
        self.user_onboard_dto = UserOnBoardDto(-1)
        SampleHeaders.__init__(self)
        AdminAuthService.__init__(self)
        AdminUserOnBoardApplicationServiceImpl.__init__(self)
        CommonSystemEnvServiceImpl.__init__(self)

    def login(self):
        if self.login_app == AppEnum.CODING.code:
            login_app_tip = "{0}({1})".format(self.login_app,
                                              self.user.company_level_login is True and "admin" or "study")
        else:
            login_app_tip = self.login_app
        try:
            user_type = self._auth()
            if user_type == UserType.account.type_name:
                return
            company_id = None
            if self.user.company:
                company_id = self.set_work_for_company_id()
            if self.login_app != AppEnum.ADMIN.code:
                self._user_onboard(company_id)
                self.user.app = self.login_app
                self._app_auth()
                if self.user.role is not None:
                    self.switch_role(self.user.role)
            else:
                self._entry_portal(company_id)
            Logger().info("The user({0}) logs in to {1} successfully.".format(self.user, login_app_tip))
        except Exception as e:
            Logger().error("The user({0}) failed to log in to {1}.".format(self.user, login_app_tip))
            raise Exception(f"Authorize Failed: {e}")

    def _auth(self):
        self.time_mills = time.time()
        encrypt_pwd = encrypt_password(self.user.password)
        jwt_authentication_request_dto = JwtAuthenticationRequest(self.user.username, encrypt_pwd)
        response = self.create_authentication_token(jwt_authentication_request_dto)
        if response is None:
            raise Exception(f"LoginParams failed: {response}")
        if response.get("jwtAuthenticationResponse") is None:
            token = response.get("token")
        else:
            token = response.get("jwtAuthenticationResponse").get("token")
        self.add_authorization(token)
        self.time_mills = time.time()
        return response.get("type")

    def _user_onboard(self, company_id):
        user_onboard_dto = UserOnBoardDto(-1)
        user_onboard_dto.workForCompanyId = company_id
        self._build_user_onboard_dto(user_onboard_dto)
        self.user_onboard_dto = user_onboard_dto
        response = self.on_board(user_onboard_dto)
        self.add_authorization(response.get("token"))

    def _build_user_onboard_dto(self, dto: UserOnBoardDto):
        onboard_system_list = self.get_user_onboard_application_list(dto.workForCompanyId)
        if not onboard_system_list:
            raise Exception("Failed to obtain onboardSystemList info.")
        for onboard_system in onboard_system_list:
            if onboard_system.get("systemName").lower() == self.login_app:
                dto.applicationId = onboard_system.get("systemId")
                self._handle_onboard_envs(onboard_system, dto)
                if self.user.company_level_login is True:
                    dto.companyLevelLogin = True
                    continue
                onboard_sponsor_list = onboard_system.get("onboardSponsorList")
                if not onboard_sponsor_list:
                    continue
                for onboard_sponsor in onboard_sponsor_list:
                    if onboard_sponsor.get("name") == self.user.sponsor:
                        dto.sponsorId = onboard_sponsor.get("sponsorId")
                        self._handle_onboard_envs(onboard_sponsor, dto)
                        onboard_study_list = onboard_sponsor.get("onboardStudyList")
                        if not onboard_study_list:
                            continue
                        for onboard_study in onboard_study_list:
                            if onboard_study.get("studyName") == self.user.study:
                                dto.studyId = onboard_study.get("studyId")
                                self._handle_onboard_envs(onboard_study, dto)
        self._assert(self.login_app, dto.applicationId, "app")
        if self.login_app in [AppEnum.EDC.code, AppEnum.DESIGN.code, AppEnum.IWRS.code, AppEnum.CODING.code]:
            if dto.companyLevelLogin is False:
                self._assert(self.user.study, dto.studyId, "study")
            else:
                self._assert(self.user.company, dto.workForCompanyId, "company")
        elif self.login_app != AppEnum.ADMIN.code:
            self._assert(self.user.sponsor, dto.sponsorId, "sponsor")
        self._assert(self.user.app_env, dto.envId, "app env")

    @staticmethod
    def _assert(attr, att_id, k):
        assert (not attr) or (False if att_id is None else True), f"Failed to obtain {k} id. {k}::{attr}."

    def _handle_onboard_envs(self, onboard_infos, dto: UserOnBoardDto):
        onboard_envs = onboard_infos.get("onboardEnvs") or list()
        for onboardEnv in onboard_envs:
            if onboardEnv.get("name") == self.user.app_env:
                dto.envId = onboardEnv.get("id")

    def set_work_for_company_id(self):
        company_id = self.get_company_id(name=self.user.company)
        if self.user.company and not company_id:
            raise Exception("The company {0} was not found.".format(self.user.company))
        return company_id

    def _app_auth(self):
        response = self.create_authentication_token()
        if response.get("jwtAuthenticationResponse") is None:
            token = response.get("token")
        else:
            token = response.get("jwtAuthenticationResponse").get("token")
        self.add_authorization(token)
        self.time_mills = time.time()

    def _entry_portal(self, company_id):
        response = self.login_portal(CrossUserUserOnBoardDto(companyId=company_id))
        try:
            token = response.get("token")
            self.add_authorization(token)
        except BaseException as e:
            print(e)



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 2024/09/06 14:40
@Description: Description
@File: auth_service.py
"""
from api_requester.core.admin.api.auth_api import AdminAuthApi
from api_requester.dto.admin.jwt_authentication_request import JwtAuthenticationRequest


class AdminAuthService(AdminAuthApi):
    """
    The AdminAuthService was generated by Generator from the AuthController.java file.
    """

    def __init__(self):
        AdminAuthApi.__init__(self)

    def create_authentication_token(self, authentication_request: JwtAuthenticationRequest = None):
        return self.create_authentication_token_api(json=authentication_request)

    def refresh_and_get_authentication_token(self):
        return self.refresh_and_get_authentication_token_api()

    def logout(self):
        return self.logout_api()

    def logout_all(self, login_name: str = None):
        return self.logout_all_api(params=dict(loginName=login_name))

    def me(self):
        return self.me_api()



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 7/24/2023 3:48 PM
@Description: Description
@File: app_url.py
"""
from api_requester.http.gateway import Gateway
from api_requester.utils.constant import AppEnum


class AppUrl(object):
    def __init__(self, app, test_env):
        self.app = app
        self.test_env = test_env

    def _app_url(self, api, app, external=False, **kwargs):
        return Gateway(self.test_env).url(app, api, external).format(**kwargs)

    def app_url(self, api, **kwargs):
        return self._app_url(api, self.app, **kwargs)

    def portal_url(self, api, **kwargs):
        return self._app_url(api, AppEnum.ADMIN.code, **kwargs)

    def external_url(self, api, **kwargs):
        return self._app_url(api, self.app, True, **kwargs)

    def which_url(self, app):
        return self.portal_url if app == AppEnum.ADMIN.code else self.app_url



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 4/29/2024 2:41 PM
@Description: Description
@File: system_env_service_impl.py
"""
from api_requester.core.common.service.system_env_service import CommonSystemEnvService
from api_requester.http.token_manager import apply_token_to_headers
from api_requester.utils.log import Logger


class CommonSystemEnvServiceImpl(CommonSystemEnvService):
    """
    CommonSystemEnvServiceImpl
    """

    def __init__(self):
        CommonSystemEnvService.__init__(self)

    def switch_role(self, role, env=None):
        payload = self.get_current_system_env()
        if payload.get("role").get("code") == role or (env is not None and payload.get("env").get("name") == env):
            return
        items = self.sponsor_env_list()
        if not items:
            return
        for item in items:
            if item.get("defaultEnv"):
                pass
            if env is None and item.get("defaultEnv") or item.get("name") == env:
                onboard_role_list = item.get("onboardRoleList")
                onboard_role = next((role_dto for role_dto in onboard_role_list if role_dto.get("name") == role), None)
                if onboard_role:
                    self.switch_env(env_id=item.get("id"), role_id=onboard_role.get("id"))
                    apply_token_to_headers(self)
                    Logger().info(f"The user switches the {role} role successfully.")
                    return



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 3/18/2024 4:59 PM
@Description: Description
@File: read_file.py
"""
import json
import os

import yaml
from lxml import etree

from api_requester.utils.log import Logger


class JSONConnector:
    def __init__(self, filepath):
        self.data = dict()
        with open(filepath, mode='r', encoding='utf-8') as f:
            self.data = json.load(f)

    @property
    def parsed_data(self):
        return self.data


class XMLConnector:
    def __init__(self, filepath):
        self.root = etree.parse(filepath).getroot()

    @property
    def parsed_root(self):
        return self.root


class MyYamlLoader(yaml.FullLoader):

    def __init__(self, stream, filepath=None):
        self.filepath = filepath and filepath or None
        super(MyYamlLoader, self).__init__(stream)


class YAMLConnector:
    def __init__(self, filepath):
        self.data = None
        with open(filepath, mode='r', encoding='utf-8') as f:
            self.data = yaml.load(f.read(), Loader=lambda s: MyYamlLoader(s, filepath))

    @property
    def parsed_data(self):
        return self.data


class SQLConnector:
    def __init__(self, filepath):
        self.data = dict()
        with open(filepath, mode='r', encoding='utf-8') as f:
            self.data = f.read()

    @property
    def parsed_data(self):
        return self.data


def connection_factory(filepath):
    if filepath.endswith('json'):
        connector = JSONConnector
    elif filepath.endswith('xml'):
        connector = XMLConnector
    elif filepath.endswith('yaml'):
        connector = YAMLConnector
    elif filepath.endswith('sql'):
        connector = SQLConnector
    else:
        raise ValueError(f'Cannot connect to {filepath}.')
    return connector(filepath)


def connect_to(filepath, ignore_error=False):
    if not os.path.exists(filepath):
        if ignore_error is False:
            raise Exception("No such file.")
        else:
            return None
    try:
        return connection_factory(filepath)
    except ValueError as ve:
        Logger().error(ve)
        if ignore_error is False:
            raise Exception(ve)
        else:
            return None



#!/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@author: xiaodong.li
@time: 11/10/2020 3:19 PM
@desc:
"""
import base64
import time

from Crypto.Cipher import PKCS1_v1_5 as Cipher_pkcs1_v1_5
from Crypto.PublicKey import RSA


def encrypt(string):
    public_key = "MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCTYnHdPs3A6JDZanoNumpZqoTam3B4yMiR" \
                 "blbaSmxGs8tW5AUEGfdevRZJn3zg/g0KETIptFXJ7oFbhYdmeo5Q8XEQnrXU1Q9GKyVZBpfJ" \
                 "ujGD7y3MaMYw29TwUdAuWDm0aWAqiwlqR2B9IWPkVBysIp2BypwfMrpe5IutObo3jQIDAQAB "
    public_key = "-----BEGIN PUBLIC KEY-----\n" + public_key + "\n-----END PUBLIC KEY-----"
    rsa_key = RSA.importKey(public_key)
    cipher = Cipher_pkcs1_v1_5.new(rsa_key)
    cipher_text = base64.b64encode(cipher.encrypt(string.encode()))
    return cipher_text.decode()


def encrypt_password(pwd):
    timestamp = int(time.time() * 1000)
    # timestamp = date_to_timestamp("2021-07-15")
    pwd_obj = dict(password=pwd, time=timestamp)
    return encrypt(repr(pwd_obj).replace("'", "\""))


def date_to_timestamp(date):
    frmt = "%Y-%m-%d"
    time_array = time.strptime(date, frmt)
    return int(time.mktime(time_array)) * 1000



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 12/30/2024 3:55 PM
@Description: Description
@File: placeholder_replacer.py
"""
import re


class PlaceholderReplacer:
    def __init__(self, replacements=None):
        self.replacements = replacements or dict()

    def add_replacement(self, placeholder, value):
        self.replacements[placeholder] = value

    def replace(self, text):
        for placeholder, value in self.replacements.items():
            text = re.sub(rf"\{{\s*{re.escape(placeholder)}\s*\}}", str(value), text)
        return text



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 12/27/2024 2:59 PM
@Description: Description
@File: call_api.py
"""
from requests import request

from api_requester.dto.admin.user_on_board_dto import UserOnBoardDto
from api_requester.dto.biz_base import BizBase
from api_requester.dto.user import EClinicalUser
from api_requester.http.app_url import AppUrl
from api_requester.http.authorize import Authorize
from api_requester.utils.placeholder_replacer import PlaceholderReplacer


class CallApi(BizBase):

    def __init__(self, username=None, password=None, sponsor=None, study=None, test_env=None, app_env=None, app=None,
                 company=None, role=None, external=False):
        u = EClinicalUser(
            username=username,
            password=password,
            sponsor=sponsor,
            study=study,
            test_env=test_env,
            app_env=app_env,
            app=app,
            company=company,
            role=role,
            external=external
        )
        super().__init__(u)
        self.headers = dict()
        self.user_onboard_dto: UserOnBoardDto = UserOnBoardDto(-1)

    def login(self):
        auth = Authorize(self.user)
        auth.login()
        self.headers = auth.headers
        self.time_mills = auth.time_mills
        self.user_onboard_dto = auth.user_onboard_dto
        return self

    def run(self, method, api, **kwargs):
        self.login()
        replacer = PlaceholderReplacer({
            "Sponsor ID": self.user_onboard_dto.sponsorId,
            "Study ID": self.user_onboard_dto.studyId,
            "Env ID": self.user_onboard_dto.envId,
            "Sponsor Name": self.user.sponsor,
            "Study Name": self.user.study,
            "Lifecycle": self.user.app_env,
        })
        url = replacer.replace(api)
        for key, value in kwargs.items():
            kwargs[key] = replacer.replace(value)
        if self.user.external is False:
            url = AppUrl(self.user.app, self.user.test_env).which_url(self.user.app)(url)
        else:
            url = AppUrl(self.user.app, self.user.test_env).external_url(url)
        request(method, url, headers=self.headers, **kwargs)



# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 2024/04/26 19:01
@Description: Description
@File: auth_api.py
"""
from api_requester.http.eclinical_requests import *


class AdminAuthApi:
    """
    The AdminAuthApi was generated by Generator from the AuthController.java file.
    """
    
    @post('/auth')
    def create_authentication_token_api(self, *args, **kwargs): ...
    
    @get('/refresh')
    def refresh_and_get_authentication_token_api(self, *args, **kwargs): ...
    
    @get('/auth/logout')
    def logout_api(self, *args, **kwargs): ...
    
    @get('/auth/all/logout')
    def logout_all_api(self, *args, **kwargs): ...
    
    @get('/me')
    def me_api(self, *args, **kwargs): ...
    
    @post('/auth/sso/token')
    def auth_sso_token_api(self, *args, **kwargs): ...
    


# !/usr/bin/python3
# -*- coding:utf-8 -*-
"""
@Author: xiaodong.li
@Time: 8/5/2024 4:35 PM
@Description: Description
@File: exceptions.py
"""
from urllib.parse import urlparse

from requests import Response


class ApiResponseException(Exception):
    def __init__(self, response: Response, message=None, exception=None, request_payload=None):
        self.status_code = response.status_code
        self.api = urlparse(response.url).path
        self.method = response.request.method
        self.message = message
        self.exception = exception
        self.request_payload = request_payload
        self.proc_code = None
        content_type = response.headers.get("content-type")
        if not content_type:
            ...
        elif "application/json" in content_type:
            self.proc_code = response.json().get("procCode")
            self.message = response.json().get("message") or message
            self.exception = response.json().get("exception") or exception
        else:
            self.message = "\n".join([message, response.text])  if message else response.text

    def __str__(self):
        messages = [
            "\nRequest response exception:",
            "Status code: {0}".format(self.status_code),
            "URL: {0} {1}".format(self.method, self.api),
        ]
        if self.request_payload:
            if isinstance(self.request_payload, dict):
                payload = "\n".join(["{0}: {1}".format(k, v) for k, v in self.request_payload.items()])
            else:
                payload = str(self.request_payload)
            messages.append("Request payload: {0}".format(payload))
        if self.proc_code:
            messages.append("Proc code: {0}".format(self.proc_code))
        if self.message:
            messages.append("Message: {0}".format(self.message))
        if self.exception:
            messages.append("Exception: {0}".format(self.exception))
        return "\n".join(messages)


class ApiValidationError(ApiResponseException):
    """自定义异常类，用于验证接口失败时引发错误"""

    def __init__(self, response: Response, message=None, exception=None, request_payload=None):
        super().__init__(response, message, exception, request_payload)


class DatabaseValidationError(Exception):
    """自定义异常类，用于数据库验证失败时引发错误"""

    def __init__(self, message):
        super().__init__(message)


class FileValidationError(Exception):
    """自定义异常类，用于文件验证失败时引发错误"""

    def __init__(self, message):
        super().__init__(message)
